@@ -31,6 +31,8 @@
 
 #include <libwps/libwps.h>
 
+#include "helper.h"
+
 using namespace libwps;
 
 #ifdef HAVE_CONFIG_H
@@ -43,19 +45,24 @@ using namespace libwps;
 
 static int printUsage()
 {
-	printf("Usage: wps2html [OPTION] <Microsoft Works or Microsoft Write text document>\n");
+	printf("`wps2html' converts various Microsoft text document formats to HTML.\n");
+	printf("Supported formats are Microsoft Works, Write and Word for DOS.\n");
+	printf("\n");
+	printf("Usage: wps2html [OPTION] FILE\n");
 	printf("\n");
 	printf("Options:\n");
-	printf("\t-e \"encoding\":   Defines the file encoding where encoding can be\n");
+	printf("\t-e ENCODING       define the file encoding, where ENCODING can be\n");
 	printf("\t\t CP037, CP424, CP437, CP737, CP500, CP775, CP850, CP852, CP855, CP856, CP857,\n");
 	printf("\t\t CP860, CP861, CP862, CP863, CP864, CP865, CP866, CP869, CP874, CP875, CP932,\n");
 	printf("\t\t CP950, CP1006, CP1026, CP1250, CP1251, CP1252, CP1253, CP1254, CP1255, CP1256,\n");
 	printf("\t\t CP1257, CP1258, MacArabic, MacCeltic, MacCEurope, MacCroation, MacCyrillic,\n");
 	printf("\t\t MacDevanage, MacFarsi, MacGaelic, MacGreek, MacGujarati, MacGurmukhi, MacHebrew,\n");
 	printf("\t\t MacIceland, MacInuit, MacRoman, MacRomanian, MacThai, MacTurkish.\n");
-	printf("\t-h:                Shows this help message\n");
-	printf("\t-p password:       Password to open the file\n");
-	printf("\t-v:                Output wps2html version \n");
+	printf("\t-h                show this help message\n");
+	printf("\t-p PASSWORD       set password to open the file\n");
+	printf("\t-v                show version information\n");
+	printf("\n");
+	printf("Report bugs to <https://sourceforge.net/p/libwps/bugs/> or <https://bugs.documentfoundation.org/>.\n");
 	return -1;
 }
 
@@ -113,18 +120,7 @@ int main(int argc, char *argv[])
 	librevenge::RVNGHTMLTextGenerator listenerImpl(document);
 	WPSResult error = WPSDocument::parse(&input, &listenerImpl, password, encoding);
 
-	if (error == WPS_ENCRYPTION_ERROR)
-		fprintf(stderr, "ERROR: Encrypted file, bad Password!\n");
-	else if (error == WPS_FILE_ACCESS_ERROR)
-		fprintf(stderr, "ERROR: File Exception!\n");
-	else if (error == WPS_PARSE_ERROR)
-		fprintf(stderr, "ERROR: Parse Exception!\n");
-	else if (error == WPS_OLE_ERROR)
-		fprintf(stderr, "ERROR: File is an OLE document, but does not contain a Microsoft Works stream!\n");
-	else if (error != WPS_OK)
-		fprintf(stderr, "ERROR: Unknown Error!\n");
-
-	if (error != WPS_OK)
+	if (libwpsHelper::checkErrorAndPrintMessage(error))
 		return 1;
 
 	printf("%s", document.cstr());
