@@ -52,15 +52,15 @@ string to_string(length_unit_t unit)
 {
     switch (unit)
     {
-        case length_unit_centimeter:
+        case length_unit_t::centimeter:
             return "centimeter";
-        case length_unit_inch:
+        case length_unit_t::inch:
             return "inch";
-        case length_unit_point:
+        case length_unit_t::point:
             return "point";
-        case length_unit_twip:
+        case length_unit_t::twip:
             return "twip";
-        case length_unit_unknown:
+        case length_unit_t::unknown:
         default:
             ;
     }
@@ -75,18 +75,21 @@ void test_measurement_conversion()
         size_t decimals;
         length_unit_t unit;
     } tests[] = {
-        { "12.34", 12.34, 2, length_unit_unknown },
-        { "35", 35, 0, length_unit_unknown },
-        { "0.69825", 0.69825, 5, length_unit_unknown },
-        { ".1592", 0.1592, 4, length_unit_unknown },
-        { "5", 5.0, 0, length_unit_unknown },
-        { "-3", -3.0, 0, length_unit_unknown },
-        { "-3.456", -3.456, 3, length_unit_unknown },
-        { "-.987", -0.987, 3, length_unit_unknown },
-        { "-100.987.", -100.987, 3, length_unit_unknown }, // Second decimal point should stop the parsing.
+        { "12.34", 12.34, 2, length_unit_t::unknown },
+        { "35", 35, 0, length_unit_t::unknown },
+        { "0.69825", 0.69825, 5, length_unit_t::unknown },
+        { ".1592", 0.1592, 4, length_unit_t::unknown },
+        { "5", 5.0, 0, length_unit_t::unknown },
+        { "-3", -3.0, 0, length_unit_t::unknown },
+        { "-3.456", -3.456, 3, length_unit_t::unknown },
+        { "-.987", -0.987, 3, length_unit_t::unknown },
+        { "-100.987.", -100.987, 3, length_unit_t::unknown }, // Second decimal point should stop the parsing.
 
-        { "12.345in", 12.345, 3, length_unit_inch },
-        { "120.30001cm", 120.30001, 5, length_unit_centimeter },
+        { "12.345in", 12.345, 3, length_unit_t::inch },
+        { "120.30001cm", 120.30001, 5, length_unit_t::centimeter },
+        { "3.12mm", 3.12, 2, length_unit_t::millimeter },
+        { "32.681pt", 32.681, 2, length_unit_t::point },
+        { "0.1234px", 0.1234, 4, length_unit_t::pixel },
     };
 
     for (size_t i = 0, n = sizeof(tests)/sizeof(tests[0]); i < n; ++i)
@@ -129,7 +132,7 @@ void test_string2number_conversion()
     {
         const char* p = tests[i].str;
         const char* p_end = p + strlen(p);
-        const char* p_parse_ended = NULL;
+        const char* p_parse_ended = nullptr;
         double converted = to_double(p, p_end, &p_parse_ended);
         cout << "original: '" << tests[i].str << "', converted: " << converted
             << ", expected: " << tests[i].expected << endl;
@@ -167,7 +170,7 @@ void test_string2long_conversion()
     {
         const char* p = tests[i].str;
         const char* p_end = p + strlen(p);
-        const char* p_parse_ended = NULL;
+        const char* p_parse_ended = nullptr;
         long converted = to_long(p, p_end, &p_parse_ended);
         cout << "original: '" << tests[i].str << "', converted: " << converted
             << ", expected: " << tests[i].expected << endl;
@@ -180,12 +183,34 @@ void test_string2long_conversion()
     }
 }
 
+void test_pstring()
+{
+    {
+        // test for trimming.
+        string s1("test"), s2("  test"), s3("   test  "), s4("test   ");
+        pstring ps1(s1.c_str()), ps2(s2.c_str()), ps3(s3.c_str()), ps4(s4.c_str());
+        assert(ps1 != ps2);
+        assert(ps1 != ps3);
+        assert(ps2 != ps3);
+        assert(ps1 != ps4);
+
+        pstring trimmed = ps1.trim();
+        assert(ps1 == trimmed); // nothing to trim.
+        assert(ps1 == ps2.trim());
+        assert(ps1 == ps3.trim());
+        assert(ps1 == ps4.trim());
+        assert(ps1.size() == ps2.trim().size());
+        assert(ps1.size() == ps3.trim().size());
+    }
+}
+
 int main()
 {
     test_date_time_conversion();
     test_measurement_conversion();
     test_string2number_conversion();
     test_string2long_conversion();
+    test_pstring();
 
     return EXIT_SUCCESS;
 }
