@@ -34,10 +34,6 @@
 #include <cstring>
 #include <sstream>
 
-#include <boost/shared_ptr.hpp>
-#include <boost/scoped_ptr.hpp>
-#include <boost/ptr_container/ptr_vector.hpp>
-
 using namespace std;
 
 namespace orcus {
@@ -198,7 +194,22 @@ bool orcus_xlsx::detect(const unsigned char* blob, size_t size)
 
 void orcus_xlsx::read_file(const string& filepath)
 {
-    mp_impl->m_opc_reader.read_file(filepath.c_str());
+    std::unique_ptr<zip_archive_stream> stream(new zip_archive_stream_fd(filepath.c_str()));
+    mp_impl->m_opc_reader.read_file(std::move(stream));
+
+    // Formulas need to be inserted to the document after the shared string
+    // table get imported, because tokenization of formulas may add new shared
+    // string instances.
+    set_formulas_to_doc();
+
+    mp_impl->mp_factory->finalize();
+}
+
+void orcus_xlsx::read_stream(const char* content, size_t len)
+{
+    std::unique_ptr<zip_archive_stream> stream(new zip_archive_stream_blob(
+                reinterpret_cast<const unsigned char*>(content), len));
+    mp_impl->m_opc_reader.read_file(std::move(stream));
 
     // Formulas need to be inserted to the document after the shared string
     // table get imported, because tokenization of formulas may add new shared
@@ -222,7 +233,7 @@ void orcus_xlsx::set_formulas_to_doc()
     xlsx_session_data::shared_formulas_type::iterator its = sdata.m_shared_formulas.begin(), its_end = sdata.m_shared_formulas.end();
     for (; its != its_end; ++its)
     {
-        xlsx_session_data::shared_formula& sf = *its;
+        xlsx_session_data::shared_formula& sf = **its;
         spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->get_sheet(sf.sheet);
         if (!sheet)
             continue;
@@ -230,7 +241,7 @@ void orcus_xlsx::set_formulas_to_doc()
         if (sf.master)
         {
             sheet->set_shared_formula(
-                sf.row, sf.column, orcus::spreadsheet::formula_grammar_xlsx_2007, sf.identifier,
+                sf.row, sf.column, orcus::spreadsheet::formula_grammar_t::xlsx_2007, sf.identifier,
                 &sf.formula[0], sf.formula.size(), &sf.range[0], sf.range.size());
         }
         else
@@ -243,7 +254,7 @@ void orcus_xlsx::set_formulas_to_doc()
     xlsx_session_data::formulas_type::iterator it = sdata.m_formulas.begin(), it_end = sdata.m_formulas.end();
     for (; it != it_end; ++it)
     {
-        xlsx_session_data::formula& f = *it;
+        xlsx_session_data::formula& f = **it;
         spreadsheet::iface::import_sheet* sheet = mp_impl->mp_factory->get_sheet(f.sheet);
         if (!sheet)
             continue;
@@ -251,13 +262,13 @@ void orcus_xlsx::set_formulas_to_doc()
         if (f.array)
         {
             sheet->set_array_formula(
-                f.row, f.column, spreadsheet::formula_grammar_xlsx_2007, &f.exp[0],
+                f.row, f.column, spreadsheet::formula_grammar_t::xlsx_2007, &f.exp[0],
                 f.exp.size(), &f.range[0], f.range.size());
         }
         else
         {
             sheet->set_formula(
-                f.row, f.column, orcus::spreadsheet::formula_grammar_xlsx_2007, &f.exp[0], f.exp.size());
+                f.row, f.column, orcus::spreadsheet::formula_grammar_t::xlsx_2007, &f.exp[0], f.exp.size());
         }
     }
 }
@@ -275,8 +286,8 @@ void orcus_xlsx::read_workbook(const string& dir_path, const string& file_name)
     if (buffer.empty())
         return;
 
-    ::boost::scoped_ptr<xml_simple_stream_handler> handler(
-        new xml_simple_stream_handler(new xlsx_workbook_context(mp_impl->m_cxt, ooxml_tokens)));
+    auto handler = orcus::make_unique<xml_simple_stream_handler>(
+        new xlsx_workbook_context(mp_impl->m_cxt, ooxml_tokens));
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -329,8 +340,10 @@ void orcus_xlsx::read_sheet(const string& dir_path, const string& file_name, xls
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
-    boost::scoped_ptr<xlsx_sheet_xml_handler> handler(
-        new xlsx_sheet_xml_handler(mp_impl->m_cxt, ooxml_tokens, data->id-1, sheet));
+
+    auto handler = orcus::make_unique<xlsx_sheet_xml_handler>(
+        mp_impl->m_cxt, ooxml_tokens, data->id-1, sheet);
+
     parser.set_handler(handler.get());
     parser.parse();
 
@@ -359,9 +372,11 @@ void orcus_xlsx::read_shared_strings(const string& dir_path, const string& file_
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
-    ::boost::scoped_ptr<xml_simple_stream_handler> handler(
-        new xml_simple_stream_handler(
-            new xlsx_shared_strings_context(mp_impl->m_cxt, ooxml_tokens, mp_impl->mp_factory->get_shared_strings())));
+
+    auto handler = orcus::make_unique<xml_simple_stream_handler>(
+        new xlsx_shared_strings_context(
+            mp_impl->m_cxt, ooxml_tokens, mp_impl->mp_factory->get_shared_strings()));
+
     parser.set_handler(handler.get());
     parser.parse();
 }
@@ -390,9 +405,11 @@ void orcus_xlsx::read_styles(const string& dir_path, const string& file_name)
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
-    ::boost::scoped_ptr<xml_simple_stream_handler> handler(
-        new xml_simple_stream_handler(
-            new xlsx_styles_context(mp_impl->m_cxt, ooxml_tokens, mp_impl->mp_factory->get_styles())));
+
+    auto handler = orcus::make_unique<xml_simple_stream_handler>(
+        new xlsx_styles_context(
+            mp_impl->m_cxt, ooxml_tokens, mp_impl->mp_factory->get_styles()));
+
     parser.set_handler(handler.get());
     parser.parse();
 }
@@ -424,8 +441,7 @@ void orcus_xlsx::read_table(const std::string& dir_path, const std::string& file
     if (buffer.empty())
         return;
 
-    boost::scoped_ptr<xlsx_table_xml_handler> handler(
-        new xlsx_table_xml_handler(mp_impl->m_cxt, ooxml_tokens, *table));
+    auto handler = orcus::make_unique<xlsx_table_xml_handler>(mp_impl->m_cxt, ooxml_tokens, *table);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -455,8 +471,7 @@ void orcus_xlsx::read_pivot_cache_def(const std::string& dir_path, const std::st
     if (buffer.empty())
         return;
 
-    boost::scoped_ptr<xlsx_pivot_cache_def_xml_handler> handler(
-        new xlsx_pivot_cache_def_xml_handler(mp_impl->m_cxt, ooxml_tokens));
+    auto handler = orcus::make_unique<xlsx_pivot_cache_def_xml_handler>(mp_impl->m_cxt, ooxml_tokens);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -465,7 +480,7 @@ void orcus_xlsx::read_pivot_cache_def(const std::string& dir_path, const std::st
     parser.parse();
 
     handler.reset();
-    mp_impl->m_opc_reader.check_relation_part(file_name, NULL);
+    mp_impl->m_opc_reader.check_relation_part(file_name, nullptr);
 }
 
 void orcus_xlsx::read_pivot_cache_rec(const std::string& dir_path, const std::string& file_name)
@@ -487,8 +502,7 @@ void orcus_xlsx::read_pivot_cache_rec(const std::string& dir_path, const std::st
     if (buffer.empty())
         return;
 
-    boost::scoped_ptr<xlsx_pivot_cache_rec_xml_handler> handler(
-        new xlsx_pivot_cache_rec_xml_handler(mp_impl->m_cxt, ooxml_tokens));
+    auto handler = orcus::make_unique<xlsx_pivot_cache_rec_xml_handler>(mp_impl->m_cxt, ooxml_tokens);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -518,8 +532,7 @@ void orcus_xlsx::read_pivot_table(const std::string& dir_path, const std::string
     if (buffer.empty())
         return;
 
-    boost::scoped_ptr<xlsx_pivot_table_xml_handler> handler(
-        new xlsx_pivot_table_xml_handler(mp_impl->m_cxt, ooxml_tokens));
+    auto handler = orcus::make_unique<xlsx_pivot_table_xml_handler>(mp_impl->m_cxt, ooxml_tokens);
 
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
@@ -528,7 +541,7 @@ void orcus_xlsx::read_pivot_table(const std::string& dir_path, const std::string
     parser.parse();
 
     handler.reset();
-    mp_impl->m_opc_reader.check_relation_part(file_name, NULL);
+    mp_impl->m_opc_reader.check_relation_part(file_name, nullptr);
 }
 
 void orcus_xlsx::read_rev_headers(const std::string& dir_path, const std::string& file_name)
@@ -553,14 +566,15 @@ void orcus_xlsx::read_rev_headers(const std::string& dir_path, const std::string
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
-    boost::scoped_ptr<xml_simple_stream_handler> handler(
-        new xml_simple_stream_handler(
-            new xlsx_revheaders_context(mp_impl->m_cxt, ooxml_tokens)));
+
+    auto handler = orcus::make_unique<xml_simple_stream_handler>(
+        new xlsx_revheaders_context(mp_impl->m_cxt, ooxml_tokens));
+
     parser.set_handler(handler.get());
     parser.parse();
 
     handler.reset();
-    mp_impl->m_opc_reader.check_relation_part(file_name, NULL);
+    mp_impl->m_opc_reader.check_relation_part(file_name, nullptr);
 }
 
 void orcus_xlsx::read_rev_log(const std::string& dir_path, const std::string& file_name)
@@ -585,9 +599,10 @@ void orcus_xlsx::read_rev_log(const std::string& dir_path, const std::string& fi
     xml_stream_parser parser(
         get_config(), mp_impl->m_ns_repo, ooxml_tokens,
         reinterpret_cast<const char*>(&buffer[0]), buffer.size());
-    boost::scoped_ptr<xml_simple_stream_handler> handler(
-        new xml_simple_stream_handler(
-            new xlsx_revlog_context(mp_impl->m_cxt, ooxml_tokens)));
+
+    auto handler = orcus::make_unique<xml_simple_stream_handler>(
+        new xlsx_revlog_context(mp_impl->m_cxt, ooxml_tokens));
+
     parser.set_handler(handler.get());
     parser.parse();
 
